package com.demo;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;

@Configuration
@PropertySource("classpath:application.properties")
public class DataSourceDbUtil {
	
	@Autowired
	private Environment env;
	
	@Bean
	public DataSource sourceDataSource() {

		  HikariConfig hikariConfig = new HikariConfig();
		    hikariConfig.setDriverClassName(env.getProperty("sourceDataSource.driver-class-name"));
		    hikariConfig.setJdbcUrl(env.getProperty("sourceDataSource.url")); 
		    hikariConfig.setUsername(env.getProperty("sourceDataSource.username"));
		    hikariConfig.setPassword(env.getProperty("sourceDataSource.password"));

		    hikariConfig.setMaximumPoolSize(10);
		    hikariConfig.setPoolName("springHikariCP");
		    hikariConfig.setMaxLifetime(300000);

		    hikariConfig.addDataSourceProperty("dataSource.cachePrepStmts", "true");
		    hikariConfig.addDataSourceProperty("dataSource.prepStmtCacheSize", "250");
		    hikariConfig.addDataSourceProperty("dataSource.prepStmtCacheSqlLimit", "2048");
		    hikariConfig.addDataSourceProperty("dataSource.useServerPrepStmts", "true");

		    HikariDataSource dataSource = new HikariDataSource(hikariConfig);

		    return dataSource;

	}
	
	
	@Bean
	public DataSource destinationDataSource() {

		  HikariConfig hikariConfig = new HikariConfig();
		    hikariConfig.setDriverClassName(env.getProperty("desticationDataSource.driver-class-name"));
		    hikariConfig.setJdbcUrl(env.getProperty("desticationDataSource.url")); 
		    hikariConfig.setUsername(env.getProperty("desticationDataSource.username"));
		    hikariConfig.setPassword(env.getProperty("desticationDataSource.password"));

		    hikariConfig.setMaximumPoolSize(10);
		    hikariConfig.setPoolName("springHikariCP");
		    hikariConfig.setMaxLifetime(300000);

		    hikariConfig.addDataSourceProperty("dataSource.cachePrepStmts", "true");
		    hikariConfig.addDataSourceProperty("dataSource.prepStmtCacheSize", "250");
		    hikariConfig.addDataSourceProperty("dataSource.prepStmtCacheSqlLimit", "2048");
		    hikariConfig.addDataSourceProperty("dataSource.useServerPrepStmts", "true");

		    HikariDataSource dataSource = new HikariDataSource(hikariConfig);

		    return dataSource;

	}
	
	
	@Bean
	public DataSource appDataSource() {

		  HikariConfig hikariConfig = new HikariConfig();
		    hikariConfig.setDriverClassName(env.getProperty("appDB.driver-class-name"));
		    hikariConfig.setJdbcUrl(env.getProperty("appDB.url")); 
		    hikariConfig.setUsername(env.getProperty("appDB.username"));
		    hikariConfig.setPassword(env.getProperty("appDB.password"));

		    hikariConfig.setMaximumPoolSize(10);
		    hikariConfig.setPoolName("springHikariCP");
		    hikariConfig.setMaxLifetime(300000);

		    hikariConfig.addDataSourceProperty("dataSource.cachePrepStmts", "true");
		    hikariConfig.addDataSourceProperty("dataSource.prepStmtCacheSize", "250");
		    hikariConfig.addDataSourceProperty("dataSource.prepStmtCacheSqlLimit", "2048");
		    hikariConfig.addDataSourceProperty("dataSource.useServerPrepStmts", "true");

		    HikariDataSource dataSource = new HikariDataSource(hikariConfig);

		    return dataSource;

	}


}
