package com.demo.controller;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.demo.service.CompareTwoTables;
import com.demo.util.MigrationJob;

@RestController
@RequestMapping("/api")
@CrossOrigin
public class MainController {

	@Autowired
	CompareTwoTables service;
	

	
	@Autowired
    private Scheduler scheduler;

	@RequestMapping("/getCompareData")
	public Map<String, Object> demo(@RequestParam("srcTable") String srcTable,
			@RequestParam("destTable") String destTable, @RequestParam("srcColumn") String srcColumn,
			@RequestParam("destColumn") String destColumn, @RequestParam("srcKey") String srcKey,
			@RequestParam("destKey") String destKey, @RequestParam int currentPageNo) throws Exception {

		System.out.println(srcTable + "-->" + destTable + "-->" + srcColumn + "-->" + destColumn);

		return service.getCompareTwoTables(srcTable, destTable, srcColumn, destColumn, srcKey, destKey, currentPageNo);

	}

	@RequestMapping("/getSrcRow")
	public Map<String, Object> getSrcRow(@RequestParam("tableName") String tableName,
			@RequestParam("columns") String columns, @RequestParam("tableKey") String tableKey,
			@RequestParam("keyValue") String keyValue) throws Exception {

		System.out.println(tableName + "-->" + columns + "-->" + tableKey);

		return service.getSrcRow(tableName, columns, tableKey, keyValue);

	}

	@RequestMapping("/getDestRow")
	public Map<String, Object> getDestRow(@RequestParam("tableName") String tableName,
			@RequestParam("columns") String columns, @RequestParam("tableKey") String tableKey,
			@RequestParam("keyValue") String keyValue) throws Exception {

		System.out.println(tableName + "-->" + columns + "-->" + tableKey);

		return service.getDestRow(tableName, columns, tableKey, keyValue);

	}

	@RequestMapping(value = "/updateSourceRowById", method = RequestMethod.POST)
	public Map<String, Object> updateSourceRowById(@RequestBody Map<String, Object> map) throws Exception {

		System.out.println(map);
		
		Map<String,Object> statusMap = new HashMap<>();
		String status = service.updateSourceRowById(map);
		
		statusMap.put("status", status);

		return statusMap;

	}

	@RequestMapping(value = "/updateDestRowById", method = RequestMethod.POST)
	public Map<String, Object> updateDestRowById(@RequestBody Map<String, Object> map) throws Exception {

		System.out.println(map);
		
		Map<String,Object> statusMap = new HashMap<>();
		String status =service.updateDestRowById(map);
		
		statusMap.put("status", status);

		return statusMap;

	}

	@RequestMapping("/dataMigration")
	public Map<String, Object> dataMigration(@RequestParam("srcTable") String srcTable,
			@RequestParam("destTable") String destTable, @RequestParam("srcColumn") String srcColumn,
			@RequestParam("destColumn") String destColumn, @RequestParam("srcKey") String srcKey,
			@RequestParam("destKey") String destKey, @RequestParam String query) throws Exception {

		System.out.println("Tables : "+srcTable + "-->" + destTable);
		System.out.println("Cols :"+ srcColumn + "-->" + destColumn);
		String status = service.dataMigration(srcTable, destTable, srcColumn, destColumn, srcKey, destKey, query);

		Map<String, Object> map = new HashMap<>();

		map.put("status", status);

		return map;

	}

	@RequestMapping("/getCategory")
	public List<Map<String, Object>> getCategory(@RequestParam String category) throws Exception {

		return service.getCategory(category);

	}

	@RequestMapping("/saveRecommendedName")
	public Map<String, Object> saveRecommendedName(@RequestBody Map<String, Object> map) throws Exception {

		System.out.println(map);

		String status = service.saveRecommendedName(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;

	}

	@RequestMapping("/getCategoryRecommendedList")
	public List<Map<String, Object>> getCategoryRecommendedList(@RequestParam String NAME) throws Exception {

		return service.getCategoryRecommendedList(NAME);

	}

	@RequestMapping("/approveCategoryValue")
	public Map<String, Object> approveCategoryValue(@RequestBody Map<String, Object> map) throws Exception {

		System.out.println(map);

		String status = service.approveCategoryValue(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;

	}

	@RequestMapping("/getCategories")
	public List<Map<String, Object>> getCategories() throws Exception {

		return service.getCategories();

	}

	@RequestMapping(value = "/getSrcTree")
	public Map<String, Object> getSrcTree() throws Exception {

		return service.getSrcTablesHierarchy();

	}

	@RequestMapping(value = "/getDestTree")
	public Map<String, Object> getDestTree() throws Exception {

		return service.getDestTablesHierarchy();

	}

	@RequestMapping("/saveRule")
	public Map<String, Object> saveRule(@RequestBody Map<String, Object> map) {

		System.out.println(map);
		String status = service.saveRule(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;
	}

	@RequestMapping("/migrateBasedOnIgnoreColumnRule")
	public Map<String, Object> migrateBasedOnIgnoreColumnRule(@RequestBody Map<String, String> map)
			throws ClassNotFoundException, SQLException {

		String status = service.migrateBasedOnIgnoreColumnRule(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;
	}

	@RequestMapping("/getRules")
	public List<Map<String, String>> getRules() throws Exception {

		return service.getRules();

	}

	@RequestMapping("/migrateBasedOnSplitRule")
	public Map<String, Object> migrateBasedOnSplitRule(@RequestBody Map<String, String> map)
			throws ClassNotFoundException, SQLException {

		String status = service.migrateBasedOnSplitRule(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;
	}

	@RequestMapping("/migrateBasedOnMergeColumnsRule")
	public Map<String, Object> migrateBasedOnMergeColumnsRule(@RequestBody Map<String, String> map)
			throws ClassNotFoundException, SQLException {

		String status = service.migrateBasedOnMergeColumnsRule(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;
	}
	
	@RequestMapping("/migrateBasedOnScalarColumnsRule")
	public Map<String, Object> migrateBasedOnScalarColumnsRule(@RequestBody Map<String, String> map)
			throws ClassNotFoundException, SQLException {

		String status = service.migrateBasedOnScalarColumnsRule(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;
	}
	
	
	@RequestMapping("/migrateBasedOnAggregateColumnsRule")
	public Map<String, Object> migrateBasedOnAggregateColumnsRule(@RequestBody Map<String, String> map)
			throws ClassNotFoundException, SQLException {

		String status = service.migrateBasedOnAggregateColumnsRule(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;
	}
	
	@RequestMapping("/getDBFunctions")
	public List<String> getDBFunctions() throws SQLException{
		
		return service.getDBFunctions();
		
	}
	
	@RequestMapping("/getMasterRules")
	public List<String> getMasterRules() throws SQLException{
		
		return service.getMasterRules();
		
	}
	
	@RequestMapping("/migrateBasedOnDefaultCaseRule")
	public Map<String, Object> migrateBasedOnDefaultCaseRule(@RequestBody Map<String, String> map)
			throws ClassNotFoundException, SQLException {

		String status = service.migrateBasedOnDefaultCaseRule(map);

		Map<String, Object> res = new HashMap<>();

		res.put("status", status);

		return res;
	}
	
	@RequestMapping("/scheduleMigration")
	public Map<String, Object> scheduleMigration(@RequestBody Map<String, String> map)
			throws Exception {

		System.out.println(map);
		
		
		
		String date = map.get("date");
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm");
		
		date = date.replace("T", " ");
		Date convertedCurrentDate = sdf.parse(date);

		JobDetail job = JobBuilder.newJob(MigrationJob.class).withIdentity("dummyJobName", "group1").build();

		Trigger trigger = TriggerBuilder.newTrigger().withIdentity("dummyTriggerName", "group1")
				.startAt(convertedCurrentDate).build();

		// schedule it
		
		scheduler.start();
		scheduler.scheduleJob(job, trigger);
		
		
		Map<String, Object> res = new HashMap<>();

		res.put("status", "Success");

		return res;
	}
	
	
	@RequestMapping("/getRulesStatus")
	public List<Map<String, String>> getRulesStatus() throws Exception {

		return service.getRulesStatus();

	}
	

}
